<?php

class Admin_Ajax_Controller extends Base_Controller
{
    /**
     * In constructor verificam sa avem o cerere Ajax
     */
    public function __construct()
    {
        parent::__construct();

        // Nu se permit decat cereri Ajax!!!
        if ( ! Request::ajax()) {
            die(__('site.no_ajax_request'));
        }
    }

    public function get_products_list()
    {
        // Tabela de unde vor fi preluate datele...
        $table = 'products';

        // Ce coloane vor fi selectate?
        $columns = array(
            'products.doctor_id',
            'u1.name AS category',
            'products.id', 
            'u2.name',
            'products.pulse_rate AS pulse_rate',
            'products.pulse_rate',
            'products.oxygen_sat AS quantity'
        );

        // Se vor face si cateva join-uri?
        $joins = array(
            array(
                'table'     => 'users AS u1',
                'tcolumn'   => 'products.doctor_id',
                'fcolumn'   => 'u1.id'
            ),
            array(
                'table'     => 'users AS u2',
                'tcolumn'   => 'products.user_id',
                'fcolumn'   => 'u2.id'
            )
        );

        // Optiuni pentru editare
        $options = array(
            'name'      => '<a href="edit/{ID}">{VALUE}</a>',
            'category'  => '<a href="../users/edit/{ID}">{VALUE}</a>',
            'price'     => '{VALUE}'
        );

        $input = Input::all();
        $items = $this->fetch_data($table, $columns, $joins, $input, $options);

        if( ! is_null($items)) {
            echo $items;
        }
    }

    /**
     * Functia verifica daca conditiile de selectare pentru actiunile in masa
     * sunt indeplinite!
     */
    public function get_check_cond()
    {
        $condition = Input::get('cond');

        // Conditiile de tip AND - si logic
        $conditions = @explode("&", $condition);

        $separators = array('=', '<', '>', '<=', '>=', '!=', '<>', 
            'LIKE', 'NOT LIKE', 'IN', 'NOT IN');

        // Pentru fiecare conditie in parte verificam sa fie bine alcatuita
        foreach($conditions as $cond) {
            if ( ! preg_match("/([a-zA-Z_])([\s]+)(" . @implode('|', $separators) . 
                    ")([\s]+)([0-9a-zA-Z_\"'%\(\)\s])/i", $cond)) {
                return 0;
            }
        }

        return 1;
    }

    private function fetch_data($table = null, $columns = null, $joins = array(), $input, $options)
    {
        // Avem ceva de procesat?
        if(empty($input)) {
            return false;
        }

        // Cate inregistrari vor fi intoarse din baza de date?
        $limit = null;
        // Incepand cu ce offset se vor prelua inregistrari din baza de date
        $offset = null;
        // Ordineaa de afisare a datelor din baza de date
        $order = array();
        // Conditiile de selectare...
        $where = array();

        // Orice input va fi filtrat...
        foreach ($input as $key => $value) {
            $input[$key] = trim(filter_var($value, FILTER_SANITIZE_STRING));
        }

        $columns_no_as = array();

        foreach ($columns as $value) {
            $columns_no_as[] = preg_replace('/\sas\s[a-z]+/', '', $value);
        }

        // Setam limita si offset-ul
        if ( ! is_null( $input['iDisplayStart']) && $input['iDisplayLength'] != '-1') {
            $offset = (int) $input['iDisplayStart'];
            $limit  = (int) $input['iDisplayLength'];
        }

        // Dupa ce coloane vom face sortarea?
        if (isset($input['iSortCol_0']))
        {
            // Cate coloane trebuie sa sortam?
            for ($i = 0; $i < (int)$input['iSortingCols']; $i++)
            {
                if ($input['bSortable_' . intval($input['iSortCol_' . $i])] == "true")
                {
                    $order[] = array(
                        'col' => $columns_no_as[$input['iSortCol_' . $i]],
                        'dir' => ($input['sSortDir_'.$i] === 'asc') ? 'asc' : 'desc'
                    );
                }
            }
        }
        
        // Alte filtrari pe care le vom aplica (de cautare in principiu)
        if (isset($input['sSearch']) && $input['sSearch'] != "")
        {
            for ($i = 0; $i < count($columns); $i++)
            {
                $where[] = array(
                    'col'  => $columns_no_as[$i],
                    'comp' => 'LIKE',
                    'val'  => '%' . $input['sSearch'] . '%'
                );
            }
        }

        // Cautare dupa coloana...
        for ($i = 0; $i < count($columns); $i++)
        {
            if (isset($input['bSearchable_' . $i]) && $input['bSearchable_' . $i] == "true" 
                && $input['sSearch_' . $i] != '')
            {
                $where[] = array(
                    'col'  => $columns_no_as[$i],
                    'comp' => 'LIKE',
                    'val'  => '%' . $input['sSearch_' . $i] .'%'
                );
            }
        }
 
        // Se creaza interogarea bazei de date folosind toate specificatiile de mai sus...
        try {
            $db = DB::table($table);

            $total  = $db->count();

            foreach ($where as $value) {
                $db->or_where($value['col'], $value['comp'], $value['val']);
            }

            foreach ($order as $value) {
                $db->order_by($value['col'], $value['dir']);
            }

            foreach ($joins as $join) {
                $db->left_join($join['table'], $join['tcolumn'], '=', $join['fcolumn']);
            }
            
            $filtered_total  = $db->count();

            $db->take($limit);
            $db->skip($offset);

            $results = $db->get($columns);
        } catch (Exception $e) {
            return null;
        }

        // Daca nu se ajunge aici inseamna ca interogarea s-a realizat cu succes
        // Trimitem rezultatul encodat JSON
        $output = array(
            "sEcho" => $input['sEcho'],
            "iTotalRecords" => $total,
            "iTotalDisplayRecords" => $filtered_total,
            "aaData" => array()
        );

        $result = array();
        for ($i = 0; $i < count($results); $i++)
        {
            $id = null;
            foreach ($results[$i] as $key => $value)
            {
                if (preg_match("/(.*)id/i", $key)) {
                    $id = $value;
                } elseif(isset($options[$key])) {
                    $result[$i][] = str_replace(array("{ID}", "{VALUE}"), array($id, $value), $options[$key]);
                } else {
                    $result[$i][] = $value;
                }
            }
        }
        
        $output['aaData'] = $result;
        
        return json_encode($output);
    }
}
