<?php

class Admin_Categories_Controller extends Base_Controller
{
    public $item_buttons = null;
    public $submenu = null;

    private $list = array();

    /**
     * Constructorul clasei
     */
    public function __construct()
    {
        parent::__construct();

        $this->submenu = Navigation::submenu('products');

        // Se genereaza butoanele...
        $this->item_buttons = Navigation::item_buttons()
            ->add_item_button(array(
            'icon' => 'icon-plus-sign',
            'link' => 'admin.categories@add',
            'text' => __('site.categories_add_title')))
            ->get_item_buttons();
    }

    /**
     * Pagina de index, va redirectiona catre lista de categorii din sistem
     *
     * @return Laravel\Redirect
     */
    public function get_index()
    {
        return Redirect::to_action('admin.categories@list');
    }

    /**
     * Functia care produce ca si iesire lista de categorii din cadru magazinului virtual
     *
     * @param null $id
     * @return Laravel\View
     */
    public function get_list($id = null)
    {
        $butons = Navigation::item_buttons()
            ->reset_item_buttons();

        // Avem categorie parinte?
        $parent = $this->retrive_parent($id);
        $parent = $parent[0]->parent;
        if ($parent > -1) {
            $butons = $butons->add_item_button(array(
                'icon' => 'icon-arrow-up',
                'link' => 'admin.categories@list/' . $parent,
                'text' => __('site.categories_go_up'),
                'class' => 'btn btn-mini'
            ));

        }
        // Se va putea edita categoria parinte?
        if ($id != null && $id > 0) {
            $butons = $butons->add_item_button(array(
                'icon' => 'icon-pencil',
                'link' => 'admin.categories@edit/' . $id,
                'text' => __('site.categories_edit_title')
            ));
        }
        $butons = $butons->add_item_button(array(
            'icon' => 'icon-plus-sign',
            'link' => 'admin.categories@add',
            'text' => __('site.categories_add_title')))
            ->get_item_buttons();

        return View::make('layout.index')
            ->nest('header', 'layout.blocks.header', array('submenu' => $this->submenu))
            ->nest('main', 'admin.categories.list', array(
            'list' => $this->fetch_categories($id),
            'item_buttons' => $butons
        ));
    }

    /**
     * Functia care afiseaza formularul de adaugare a unei categorii in sistem
     *
     * @return Laravel\View
     */
    public function get_add()
    {
        $this->list_categories();

        return View::make('layout.index')
            ->nest('header', 'layout.blocks.header', array(
            'submenu' => $this->submenu
        ))
            ->nest('main', 'admin.categories.add', array(
            'item_buttons' => $this->item_buttons,
            'parents' => $this->list
        ));
    }

    /**
     * Functia adauga o noua categorie in sistem
     *
     * @return Laravel\Redirect|Laravel\View
     */
    public function post_add()
    {
        // A fost trimis formularul pentru "submit"?
        if (Input::get('submit')) {
            $rules = array(
                'catname' => 'required|max:64|unique:categories,name',
                'catparent' => 'required',
                'catdesc' => 'required|max:256',
                'catimage' => 'image|max:' . Config::get('application.maxsize') . '|mimes:' . Config::get('application.extension')
            );

            $input = Input::all();
            $validation = Validator::make($input, $rules);

            if ($validation->fails()) {
                Vsession::cadd('r', $validation->errors->first())->cflash('status');
            } else {
                $cat['catname'] = Input::get('catname');
                $cat['catdesc'] = Input::get('catdesc');

                foreach ($cat as $key => $value) {
                    $cat[$key] = ($value !== '') ? trim(filter_var($value, FILTER_SANITIZE_STRING)) : null;
                }

                try {
                    $extension = File::extension(Input::file('catimage.name'));
                    if ($extension) {
                        // Generam un string aleator
                        $file = Str::random(16, 'alpha') . '.' . $extension;

                        // Unde vom scrie datele?
                        $path = Config::get('application.upload_path') . DS . 'images' . DS . 'categories' . DS;

                        // Incarcam imaginea pe server
                        $result = Input::upload('catimage', $path, $file);

                        // Daca nu s-a putut incarca vom returna o eroare
                        if (!$result) {
                            Vsession::cadd('r', __('site.categories_add_upload'))->cflash('status');
                            return Redirect::to_action('admin.categories@add');
                        }
                    } else {
                        $file = 'default.png';
                    }

                    $id = DB::table('categories')->insert_get_id(array(
                            'parent' => Input::get('catparent'),
                            'name' => $cat['catname'],
                            'description' => $cat['catdesc'],
                            'image' => $file)
                    );
                } catch (Exception $e) {
                    Vsession::cadd('r', __('site.categories_add_error'))->cflash('status');
                    return Redirect::to_action('admin.categories@add');
                }

                // Stergem cache-ul cu arborele de categorii
                Cache::forget('categories_tree.cache');

                // Daca totul e in regula afisam un mesaj specific
                Vsession::cadd('g', __('site.categories_add_success'))->cflash('status');
                return Redirect::to_action('admin.categories@list');
            }
        }

        return $this->get_add();
    }

    /**
     * Functia care afiseaza formularul de editare a unei categorii din sistem
     *
     * @param null $id
     * @param null $category
     * @return Laravel\Redirect|Laravel\View
     */
    public function get_edit($id = null, $category = null)
    {
        // Verificam sa avem un id specificat
        if ($id != null) {
            // Ne asiguram ca avem un numar intreg (securizare impotriva SQL Injection)
            $id = trim(filter_var($id, FILTER_SANITIZE_NUMBER_INT));
        } else {
            Redirect::to_action('admin.categories@list');
        }

        // Obtinem detaliile categoriei curente
        if ($category == null) {
            $category = $this->fetch_category($id);
            if ($category == null) {
                return Redirect::to_action('admin.categories@list');
            }
        }

        // Avem nevoie din nou de lista de categorii
        $this->list_categories();

        // Generam butoanele
        $this->item_buttons = Navigation::item_buttons()
            ->reset_item_buttons()
            ->add_item_button(array(
            'icon' => 'icon-minus-sign icon-white',
            'link' => 'admin.categories@delete/' . $id,
            'text' => __('site.categories_delete_title'),
            'class' => 'btn-danger delete',
        ))->get_item_buttons();

        return View::make('layout.index')
            ->nest('header', 'layout.blocks.header', array(
            'submenu' => $this->submenu,
        ))
            ->nest('main', 'admin.categories.edit', array(
            'category' => $category,
            'parents' => $this->list,
            'item_buttons' => $this->item_buttons
        ));
    }

    /**
     * Functia realizeaza prelucrarile necesare atunci cand un formular de editare a unei
     * categorii este trimis cu POST
     *
     * @param null $id
     * @return Laravel\Redirect|Laravel\View
     */
    public function post_edit($id = null)
    {
        $category = null;

        if (Input::get('submit')) {
            // Id-ul este unul corect?
            if ($id != null) {
                $id = trim(filter_var($id, FILTER_SANITIZE_NUMBER_INT));
            } else {
                Redirect::to_action('admin.categories@list');
            }

            $category = $this->fetch_category($id);
            if ($category == null) {
                return Redirect::to_action('admin.categories@list');
            }

            $catname = '';
            // Am ales sa modificam numele categoriei?
            if (Input::get("catname") != '' && Input::get("catname") != $category->name) {
                // Trebuie sa ne asiguram ca noul nume nu mai exista in baza de date
                $catname = '|unique:categories,name';
            }

            // In fine, avem vectorul final de reguli
            $rules = array(
                'catname' => 'max:64' . $catname,
                'catdesc' => 'max:256',
                'catimage' => 'image|max:' . Config::get('application.maxsize') . '|mimes:' . 
                    Config::get('application.extension')
            );

            $input = Input::all();

            $validation = Validator::make($input, $rules);

            if ($validation->fails()) {
                Vsession::cadd('r', $validation->errors->first())->cflash('status');
            } else {
                foreach ($input as $key => $value) {
                    $input[$key] = ($value !== '') ? trim(filter_var($value, FILTER_SANITIZE_STRING)) : null;
                }

                // Verificam sa nu avem categoria parinte identica cu categoria noastra
                if ($id == $input['catparent']) {
                    Vsession::cadd('r', __('site.categories_edit_error1'))->cflash('status');
                    return $this->get_edit($id, $category);
                }

                try {
                    // A fost modificata numele categoriei?
                    if ($input['catname'] != '' && $input['catname'] != $category->name) {
                        $category->name = $input['catname'];
                    }

                    // A fost modificata descrierea categoriei?
                    if ($input['catdesc'] != '' && $input['catdesc'] != $category->description) {
                        $category->name = $input['catdesc'];
                    }

                    // A fost modificata categoria parinte?
                    if ($input['catparent'] != $category->parent) {
                        $category->parent = $input['catparent'];
                    }

                    // S-a modificat si imaginea?
                    $extension = File::extension(Input::file('catimage.name'));
                    $path = Config::get('application.upload_path') . DS . 'images' . DS . 'categories' . DS;

                    if ($extension) {
                        // Generam un string aleator
                        $file = Str::random(16, 'alpha') . '.' . $extension;

                        // Incarcam imaginea pe server
                        $result = Input::upload('catimage', $path, $file);

                        // Daca nu s-a putut incarca vom returna o eroare
                        if (!$result) {
                            Vsession::cadd('r', __('site.categories_add_upload'))->cflash('status');
                            return $this->get_edit($id, $category);
                        }
                    } else {
                        $file = null;
                    }

                    // Am urcat noua imagine pe server... Ce facem mai departe?
                    if ($file != null) {
                        // Stergem vechea imagine de pe disc
                        File::delete($path . $category->image);
                        // Dupa care actualizam noua imagine
                        $category->image = $file;
                    }

                    // Actualizam datele in baza de date
                    DB::table('categories')
                        ->where_id($id)
                        ->update(array(
                        'name' => $category->name,
                        'description' => $category->description,
                        'parent' => $category->parent,
                        'image' => $category->image
                    ));

                    // Reseteam cache-ul cu arborele de categorii
                    Cache::forget('categories_tree.cache');
                } catch (Exception $e) {
                    Vsession::cadd('r', __('site.categories_edit_error2'))->cflash('status');
                    return $this->get_edit($id, $category);
                }

                Vsession::cadd('g', __('site.categories_edit_success'))->cflash('status');
                return $this->get_edit($id, $category);
            }
        }

        return $this->get_edit($id, $category);
    }

    /**
     * Functia realizeaza stergerea unei categorii utilizand id-ul ei
     *
     * @param null $id
     * @return Laravel\Redirect
     */
    public function get_delete($id = null)
    {
        // Avem un id valid?
        if ($id != null) {
            $id = trim(filter_var($id, FILTER_SANITIZE_NUMBER_INT));
        } else {
            return Redirect::to_action('admin.categories@list');
        }

        // Are categoria specificata subcategorii
        $categories = Category::where('parent', '=', $id)->take(1)->count();
        if ($categories > 0) {
            Vsession::cadd('r', __('site.categories_delete_error1'))->cflash('status');
            return Redirect::to_action('admin.categories@list');
        }

        // Are categoria aceasta produse?
        if ($this->has_products($id) > 0) {
            Vsession::cadd('r', __('site.categories_delete_error2'))->cflash('status');
            return Redirect::to_action('admin.categories@list');
        }

        // Daca am ajuns aici inseamna ca ar trebui sa incercam sa stergem categoria
        if (Category::find($id)->delete()) {
            Vsession::cadd('g', __('site.categories_delete_success'))->cflash('status');
            // Stergem cache-ul cu arborele de categorii
            Cache::forget('categories_tree.cache');
        } else {
            Vsession::cadd('r', __('site.categories_delete_error3'))->cflash('status');
        }

        return Redirect::to_action('admin.categories@list');
    }

    /**
     * Functia produce listarea categoriilor tinand cont de arborele de corespondenta
     */
    private function list_categories()
    {
        $this->list[0] = __('site.categories_add_noparent');

        $tree = Cache::get('categories_tree.cache');
        // Daca intrarea nu exista in cache o vom pune...
        if ($tree == null) {
            // O vom obtine, dupa care o punem in cache
            $categories = $this->fetch_parents();

            // Construim arborele si il punem in cache
            $tree = $this->build_tree($categories);
            Cache::forever('categories_tree.cache', $tree);
        }

        // Construim lista finala...
        $this->build_list($tree, '');
    }

    /**
     * Functia intoarce id-ul parintelui unei categorii din baza de date
     *
     * @param null $id
     * @return int : Se returneaza -1 daca respectiva categorie nu are un parinte
     */
    private function retrive_parent($id = null)
    {
        if ($id == null) {
            return -1;
        }

        $ids = Category::where('id', '=', (int)$id)->take(1)->get(array('parent'));

        return $ids;
    }

    /**
     * Obtinem toate categoriile care fac parte dintr-o anumita categorie parinte
     *
     * @param null $id : Id-ul categoriei parinte
     * @return array
     */
    private function fetch_categories($id = null)
    {
        if ($id == null) {
            $id = 0;
        }

        $categories = Category::where('parent', '=', (int)$id)->get(array('id', 'name', 'description'));

        return $categories;
    }

    /**
     * Construieste arborele de categorii existente in cadrul magazinului
     *
     * @param $categories
     * @return mixed
     */
    private function build_tree(&$categories)
    {

        $map = array(
            0 => array('sub' => array())
        );

        foreach ($categories as &$category) {
            $category['sub'] = array();
            $map[$category['id']] = & $category;
        }

        foreach ($categories as &$category) {
            $map[$category['parent']]['sub'][] = & $category;
        }

        return $map[0]['sub'];
    }

    /**
     * Functia construieste o lista de categorii, in functie de dependenta dintre ele
     *
     * @param $tree : Arborele de categorii
     * @param $indent : Identare recursiva
     */
    private function build_list($tree, $indent)
    {
        foreach ($tree as $subtree) {
            $this->list[$subtree['id']] = $indent . $subtree['name'];
            if (count($subtree['sub']) > 0) {
                $this->build_list($subtree['sub'], $indent . ' - ');
            }
        }
    }

    /**
     * Arborele de categorii este construit pe baza informatiilor preluate din baza de date
     * de catre aceasta metoda privata
     *
     * @return array
     */
    private function fetch_parents()
    {
        $categories = Category::get(array('id', 'parent', 'name'));

        $cats = array();
        foreach ($categories as $category) {
            $cats[] = array('id' => $category->id, 'parent' => $category->parent, 'name' => $category->name);
        }

        return $cats;
    }

    /**
     * Intoarcem informatiile legate de o categorie (functie necesara pentru editare)
     * @param $id
     * @return mixed
     */
    private function fetch_category($id)
    {
        $category = Category::find($id);

        return $category;
    }

    /**
     * Metoda intoarce numarul de produse ce fac parte dintr-o anumita categorie.
     *
     * @param $id
     * @return int
     */
    private function has_products($id)
    {
        $count = DB::table('products')
            ->where('categories_id', '=', $id)
            ->take(1)->count();

        return $count;
    }
}
?>

