<?php

class Admin_Results_Controller extends Base_Controller
{
    public $item_buttons = null;
    public $submenu = null;

    private $list = array();

    /**
     * Constructorul acestei clase...
     */
    public function __construct()
    {
        parent::__construct();

        $this->submenu = Navigation::submenu('results'); 

        Validator::register('doctor_exists', function($attribute, $value, $parameters)
        {
            if ($value == 'null') {
                return false;
            }

            $doctor = DB::table('users')->where_id($value)->first();
            
            if ($doctor !== null) {
                return true;
            } else {
                return false;
            }
        });
    }

    /**
     * Actiunea implicita consta in listarea tuturor produselor din magazin
     */
    public function get_index()
    {
        return Redirect::to_action('admin.results@list');
    }

    /**
     * Metoda produce listarea tuturor produselor din magazinul virtual
     */
    public function get_list()
    {
        // Generam butoanele care vor fi afisate
        $this->item_buttons = Navigation::item_buttons()
        	->add_item_button(array(
            	'icon' => 'icon-tasks',
                'link' => 'admin.results@mass',
                'text' => __('site.results_mass_title')
        	))->add_item_button(array(
                'icon' => 'icon-plus-sign',
                'link' => 'admin.results@add',
                'text' => __('site.results_add_title')
            ))->get_item_buttons();

        $list = array();

        return View::make('layout.index')
        	->nest('header', 'layout.blocks.header', array(
            	'submenu' 		=> $this->submenu,
            ))->nest('main', 'admin.results.list', array(
            	'list'   		=> $list,
                'item_buttons' 	=> $this->item_buttons
            ));
    }

    /**
     * Functia ofera posibilitatea de adaugare a unui produs in sistem.
     */
    public function get_add()
    {
        return View::make('layout.index')
            ->nest('header', 'layout.blocks.header', array(
                'submenu'       => $this->submenu,
            ))->nest('main', 'admin.results.add', array(
                'doctors'       => $this->list_data('users', 'role', 2),
                'item_buttons'  =>  $this->item_buttons
            ));
    }

    /**
     * Functia realizeaza adaugarea propriu-zisa a unui produs in baza de
     * date a magazinului virtual.
     */
    public function post_add()
    {
        if (Input::get('submit'))
        {
            $rules = array(
                'pulse_rate'        => 'required|numeric',
                'oxygen_sat'        => 'required|numeric',
                'doctor'            => 'required|doctor_exists'
            );

            $input = Input::all();
            $validation = Validator::make($input, $rules);

            if ($validation->fails())
            {
                Vsession::cadd('r',  $validation->errors->first())->cflash('status');
            }
            else
            {
                foreach ($input as $key => $value) {
                    $input[$key] = ($value !== '') ? trim(filter_var($value, FILTER_SANITIZE_STRING)) : null;
                }

                // Se adauga intrarea in baza de date...
                $id = DB::table('products')->insert_get_id(array(
                    'pulse_rate'        => $input['pulse_rate'],
                    'oxygen_sat'        => $input['oxygen_sat'],
                    'doctor_id'         => $input['doctor'],
                    'user_id'           => Auth::user()->id,
                    'timestamp'         => time()                   
                ));

                // Afisam un mesaj de succes!
                Vsession::cadd('g', __('site.results_add_success'))->cflash('status');

                return Redirect::to_action('admin.results@list');
            }
        }

        return $this->get_add();
    }

    public function get_edit($id = null)
    {
        // Verificam daca produsul dorit spre a fi editat exista in baza de date
        if( ! $id || ! $this->item_exists($id, 'products')) {
            return Redirect::to_action('admin.results@list');
        }

        $id = trim(filter_var($id, FILTER_SANITIZE_NUMBER_INT));

        // Generam butoanele care vor fi afisate
        $this->item_buttons = Navigation::item_buttons()
            ->add_item_button(array(
                'icon'  => 'icon-minus-sign icon-white',
                'link'  => 'admin.results@delete/' . $id,
                'text'  => __('site.results_delete_title'),
                'class' => 'btn-danger delete',
            ))->get_item_buttons();

        return View::make('layout.index')
            ->nest('header', 'layout.blocks.header', array(
                'submenu' => $this->submenu,
            ))->nest('main', 'admin.results.edit', array(
                'doctors'       => $this->list_data('users', 'role', 2),
                'item'          => $this->fetch_item($id),
                'item_buttons'  => $this->item_buttons
            ));
    }

    public function post_edit($id = null)
    {

        if(Input::get('submit'))
        {

            $id = trim(filter_var($id, FILTER_SANITIZE_NUMBER_INT));

            if( ! $id || (($result = $this->fetch_item($id)) == null)) {
                return Redirect::to_action('admin.results@list');
            }

            $rules = array(
                'pulse_rate'        => 'required|numeric',
                'oxygen_sat'        => 'required|numeric',
                'doctor'            => 'required|doctor_exists'
            );

            $input = Input::all();
            $validation = Validator::make($input, $rules);

            if($validation->fails())
            {
                Vsession::cadd('r',  $validation->errors->first())->cflash('status');
            }
            else
            {
                foreach ($input as $key => $value) {
                    $input[$key] = ($value !== '') ? trim(filter_var($value, FILTER_SANITIZE_STRING)) : null;
                }

                $update = array();

                // Ce actualizari se vor face?
                if ($input['pulse_rate'] != '' && $input['pulse_rate'] != $result->pulse_rate) {
                    $update['pulse_rate'] = $input['pulse_rate'];
                }
                if ($input['oxygen_sat'] != '' && $input['oxygen_sat'] != $result->oxygen_sat) {
                    $update['oxygen_sat'] = $input['oxygen_sat'];
                }
                if ($input['doctor'] != $result->doctor_id) {
                    $update['doctor_id'] = $input['doctor'];
                }

                // Se face o actualizare la cel putin o coloana?
                if (count($update) > 0) {
                    DB::table('products')
                        ->where('id', '=', $id)
                        ->update($update);
                }

                // Produsul a fost editat cu succes!
                Vsession::cadd('g', __('site.results_edit_success'))->cflash('status');
                return Redirect::to_action('admin.results@edit/' . $id);
            }
        }

        return $this->get_edit($id);
    }

    /**
     * Functia realizeaza stergerea unui produs din sistem.
     */
    public function get_delete($id = null)
    {
        if($id != null) {
            $id = trim(filter_var($id, FILTER_SANITIZE_NUMBER_INT));
        } else {
            return Redirect::to_action('admin.results@list');
        }

        // Se realizeaza stergerea efectiva
        DB::table('products')->delete($id);

        // Actiune realizata cu succes
        Vsession::cadd('g',  __('site.results_delete_success'))->cflash('status');
        return Redirect::to_action('admin.results@list');
    }

    /**
     * Functie necesara pentru executarea unor actiuni in masa!
     */
    public function get_mass()
    {
        return View::make('layout.index')
            ->nest('header', 'layout.blocks.header', array(
                'submenu'       => $this->submenu,
            ))->nest('main', 'admin.results.mass', array(
                'actions'       => array(
                    0 => __('site.results_mass_act1'),
                    1 => __('site.results_mass_act2')
                ),
                'doctors'       => $this->list_data('users', 'role', 2),
                'item_buttons'  => $this->item_buttons
            ));
    }

    /**
     * Functie necesara pentru procesarea unor actiuni in masa alese de administrator!
     */
    public function post_mass()
    {

        if(Input::get('submit'))
        {
            Validator::register('sql_condition', function($attribute, $value, $parameters)
            {
                // Conditiile de tip AND - si logic
                $conditions = @explode("&", $value);

                $separators = array('=', '<', '>', '<=', '>=', '!=', '<>', 
                    'LIKE', 'NOT LIKE', 'IN', 'NOT IN');

                // Pentru fiecare conditie in parte verificam sa fie bine alcatuita
                foreach($conditions as $cond) {
                    if ( ! preg_match("/([a-zA-Z_])([\s]+)(" . @implode('|', $separators) . 
                        ")([\s]+)([0-9a-zA-Z_\"'%\(\)\s])/i", $cond)) {
                        return false;
                    }
                }

                return true;
            });

            $rules = array();

            $action = Input::get('mycondition');
            if ($action == 0) {
                $rules['source'] = 'required|doctor_exists';
                $rules['destination'] = 'required|doctor_exists|different:source';
            }

            $rules['mycondition'] = 'sql_condition';

            $input = Input::all();
            $validation = Validator::make($input, $rules);

            if($validation->fails())
            {
                Vsession::cadd('r',  $validation->errors->first())->cflash('status');
            }
            else
            {
                foreach ($input as $key => $value) {
                    $input[$key] = ($value !== '') ? trim(filter_var($value, FILTER_SANITIZE_STRING)) : null;
                }

                $where = '';
                if ($input['mycondition'] != '') {
                    $where = "WHERE " . $input['mycondition'];
                }

                switch ($input['myaction']) {
                    case 0:
                        $s = (int) $input['source'];
                        $d = (int) $input['destination'];
                        if ($where != '') {
                            $where .= " AND docotor_id = $s";
                        } else {
                            $where .= "WHERE doctor_id = $s";
                        }

                        // Mutam produsele
                        DB::query("UPDATE products SET doctor_id = $d $where");
                        break;
                    default:
                        // Stergem o serie de intrari
                        DB::query("DELETE FROM products $where");
                }

                // Produsul a fost editat cu succes!
                Vsession::cadd('g', __('site.results_mass_success'))->cflash('status');
                return Redirect::to_action('admin.results@list');
            }
        }

        return $this->get_mass();
    }

    /**
     * Functia ne ajuta sa determinam categoriile.
     */
    private function list_data($table, $whereField=null, $whereValue=null, $whereSign='=')
    {
        // Adaugam si o optiune implicita...
        $list_null = array('null' => __('site.choose_something'));
        $list = array();

        // Procesam datele...
        if ($whereField != null && $whereValue != null)
            $data = DB::table($table)->where($whereField, $whereSign, $whereValue)->get(array('id', 'name'));
        else
            $data = DB::table($table)->get(array('id', 'name'));
        foreach ($data as $d) {
            $list[$d->id] = $d->name;
        }

        // Realizam un merge pe cei doi vectori
        $list = (array) $list_null + (array) $list;

        return $list;
    }

    /**
     * Functia intoarce o serie de informatii legate de un anumit produs.
     */
    private function fetch_item($id)
    {

        $data = DB::table('products')
            ->where_id($id)
            ->first();

        return $data;
    }

    /**
     * Functia verifica daca un element exista intr-o anumita tabela a bazei de
     * date.
     */
    private function item_exists($id, $table)
    {
        $data = DB::table($table)->find($id);

        return ($data !== null) ? true : false;
    }

    /**
     * Functia produce listarea categoriilor tinand cont de arborele de corespondenta
     */
    private function list_categories($noparent = false)
    {
        if ( ! $noparent) {
            $this->list[0] = __('site.categories_add_noparent');
        }

        $tree = Cache::get('categories_tree.cache');
        // Daca intrarea nu exista in cache o vom pune...
        if ($tree == null) {
            // O vom obtine, dupa care o punem in cache
            $categories = $this->fetch_parents();

            // Construim arborele si il punem in cache
            $tree = $this->build_tree($categories);
            Cache::forever('categories_tree.cache', $tree);
        }

        // Construim lista finala...
        $this->build_list($tree, '');
    }

    /**
     * Construieste arborele de categorii existente in cadrul magazinului
     *
     * @param $categories
     * @return mixed
     */
    private function build_tree(&$categories)
    {

        $map = array(
            0 => array('sub' => array())
        );

        foreach ($categories as &$category) {
            $category['sub'] = array();
            $map[$category['id']] = & $category;
        }

        foreach ($categories as &$category) {
            $map[$category['parent']]['sub'][] = & $category;
        }

        return $map[0]['sub'];
    }

    /**
     * Functia construieste o lista de categorii, in functie de dependenta dintre ele
     *
     * @param $tree : Arborele de categorii
     * @param $indent : Identare recursiva
     */
    private function build_list($tree, $indent)
    {
        foreach ($tree as $subtree) {
            $this->list[$subtree['id']] = $indent . $subtree['name'];
            if (count($subtree['sub']) > 0) {
                $this->build_list($subtree['sub'], $indent . ' - ');
            }
        }
    }

    /**
     * Arborele de categorii este construit pe baza informatiilor preluate din baza de date
     * de catre aceasta metoda privata
     *
     * @return array
     */
    private function fetch_parents()
    {
        $categories = Category::get(array('id', 'parent', 'name'));

        $cats = array();
        foreach ($categories as $category) {
            $cats[] = array('id' => $category->id, 'parent' => $category->parent, 
                'name' => $category->name);
        }

        return $cats;
    }

}
