<?php 

class Admin_Settings_Controller extends Base_Controller
{
    public $submenu = null;

    private $load_settings = false;
    private $settings = null;

    public function __construct()
    {
        parent::__construct();

        $this->submenu = Navigation::submenu('products');
    }

    public function get_index()
    {
        return View::make('layout.index')
            ->nest('header', 'layout.blocks.header', array('submenu' => $this->submenu))
            ->nest('main', 'admin.settings.index', array());
    }

    public function post_index()
    {
        if(Input::get('submit'))
        {   
            // Inregistram un validator pentru limba aleasa...
            Validator::register('language_exists', function($attribute, $value, $parameters)
            {
                if(array_key_exists($value, Config::get('site.languages'))) {
                    return true;
                } else {
                    return false;
                }
            });

            // Care sunt regulile pentru modificarea unor setari
            $rules = array(
                'language'      => 'language_exists',
                'siteemail'     => 'email',
                'maxsize'       => 'integer',
                'maxresize'     => 'match:/([0-9]+)x([0-9]+)/'
            );


            $input = Input::all();
            $input['resize'] = Input::get('resize', '0');
            $validation = Validator::make($input, $rules);

            if($validation->fails()) {
                Vsession::cadd('r',  $validation->errors->first())->cflash('status');
            } else {
                $i = 0;
                // Incercam sa actualizam fiecare setare modificata, in parte
                foreach ($input as $field => $value)
                {
                    if (empty($value) || ! $this->update_needed($field, $value)) {
                        continue;
                    }

                    $value = trim(filter_var($value, FILTER_SANITIZE_STRING));

                    DB::table('settings')
                        ->where_field($field)
                        ->take(1)
                        ->update(array('value' => $value));

                    $i++;
                }

                // Actualizam datele din cache doar daca s-a modificat cel putin o setare din baza de date
                $this->update_cache($i);

                // S-au putut realiza actualizarile cu succes?
                Vsession::cadd('g', __('site.setting_success'))->cflash('status');

                // Realizam o redirectionare catre pagina principala cu setari
                return Redirect::to_action('admin.settings');
            }
        }

        return $this->get_index();
    }

    /**
     * Functia verifica daca o intrare de setare din cache necesita sau nu actualizare
     *
     * @param $field
     * @param $value
     * @return bool
     */
    private function update_needed($field, $value)
    {
        // Au fost incarcate setarile din cache?
        if ( ! $this->load_settings) {
            $this->settings = Cache::get('settings.cache');
            $this->load_settings = true;
        }

        // Daca nu s-a putut citi din cache, nu avem cu ce compara, deci returnam "true" intotdeauna
        if ($this->settings == null) {
            return true;
        }

        // Cautam setarea in lista de setari si vedem daca necesita actualizare
        $total = count($this->settings);
        for($i = 0; $i < $total; $i++) {
            if ($this->settings[$i]->field == $field) {
                // Necesita actualizare si in baza de date
                if ($this->settings[$i]->value != $value) {
                    $this->settings[$i]->value = $value;
                    return true;
                } else {
                    // Oprim cautarea (salvam timp)
                    break;
                }
            }
        }

        // Daca ajungem aici inseamna ca setarea nu necesita actualizare
        return false;
    }

    /**
     * Functia verifica daca trebuie actualizat sau nu cache-ul intern de setari, in urma modificarii
     * cel putin a unei setari din interfata de administrare.
     *
     * @param $i : Numarul de setari modificate in baza de date
     */
    private function update_cache($i)
    {
        if ($i > 0 && $this->settings != null) {
            // Suprascriem datele existente din cache
            Cache::forever('settings.cache', $this->settings);
        }
    }
}

?>