<?php

class Admin_Users_Controller extends Base_Controller
{
    public $item_buttons = null;
    public $submenu = null;

    public function __construct()
    {
        parent::__construct();

        $this->submenu = Navigation::submenu('results');

        // Generam butoanele
        $this->item_buttons = Navigation::item_buttons()
            ->add_item_button(array(
            'icon' => 'icon-plus-sign',
            'link' => 'admin.users@add',
            'text' => __('site.user_add_title')))
            ->get_item_buttons();
    }

    public function get_index()
    {
        return Redirect::to_action('admin.users@list');
    }

    public function get_list()
    {
        return View::make('layout.index')
            ->nest('header', 'layout.blocks.header', array(
            'submenu' => $this->submenu,
        ))
            ->nest('main', 'admin.users.list', array(
            'list' => $this->fetch_users(),
            'item_buttons' => $this->item_buttons
        ));
    }

    public function get_add()
    {
        return View::make('layout.index')
            ->nest('header', 'layout.blocks.header', array('submenu' => $this->submenu))
            ->nest('main', 'admin.users.add', array('item_buttons' => $this->item_buttons));
    }

    public function post_add()
    {
        if (Input::get('submit')) {
            // Inregistram un nou validator
            Validator::register('role_test', function ($attribute, $value, $parameters) {
                return ($value > -1 && $value < 3) ? true : false;
            });

            // Care sunt regulile de validare?
            $rules = array(
                'name' => 'required|max:256',
                'email' => 'required|email|unique:users',
                'username' => 'required|max:32|unique:users',
                'password' => 'required|min:6|max:32',
                'role' => 'required|role_test'
            );

            $input = Input::all();
            $validation = Validator::make($input, $rules);

            // A aparut vreo eroare?
            if ($validation->fails()) {
                Vsession::cadd('r', $validation->errors->first())->cflash('status');
            } else {
                // Curatam datele care vor fi introduse in baza de date
                foreach ($input as $key => $value) {
                    $input[$key] = ($value !== '') ? trim(filter_var($value, FILTER_SANITIZE_STRING)) : null;
                }

                // Creaza un nou membru
                $user = new \Verify\Models\User;
                $user->username = $input['username'];
                $user->name = $input['name'];
                $user->email = $input['email'];
                $user->password = $input['password'];
                $user->role = $input['role'];
                $user->verified = 1;
                $user->save();

                // Am creat membrul cu succes
                Vsession::cadd('g', __('site.user_add_success'))->cflash('status');

                // Redirectionam administratorul catre pagina cu lista de membri
                return Redirect::to_action('admin.users@list');
            }
        }

        // Reusing view
        return $this->get_add();
    }

    public function get_edit($id = null, $user = null)
    {
        // Ce date am primit?
        if ($id == null || !$this->user_exists($id)) {
            return Redirect::to_action('admin.users@list');
        }

        // Filtram id-ul pentru a obtine securitate
        $id = trim(filter_var($id, FILTER_SANITIZE_NUMBER_INT));

        // Obtinem utilizatorul
        if (is_null($user)) {
            $user = $this->fetch_user($id);
            $user = $user[0];
        }

        // Generam butoanele
        $this->item_buttons = Navigation::item_buttons()
            ->add_item_button(array(
            'icon' => 'icon-minus-sign icon-white',
            'link' => 'admin.users@delete/' . $id,
            'text' => __('site.user_delete_title'),
            'class' => 'btn-danger delete',
        ))
            ->get_item_buttons();

        return View::make('layout.index')
            ->nest('header', 'layout.blocks.header', array(
            'submenu' => $this->submenu,
        ))
            ->nest('main', 'admin.users.edit', array(
            'user' => $user,
            'item_buttons' => $this->item_buttons
        ));
    }

    public function post_edit($id = null)
    {
        // Verificam daca utilizatorul specificat exista in sistem?
        if ($id == null || !$this->user_exists($id)) {
            return Redirect::to_action('admin.users@list');
        }

        // Obtinem informatiile despre utilizator
        $user = Auth::retrieve($id);

        // Daca totul e in regula, purcedem la editarea utilizatorului
        if (Input::get('submit')) {
            // Reguli de validare acceptate
            $rules = array(
                'name' => 'max:256',
                'password' => 'min:6|max:32'
            );

            // S-a moficat adresa de email?
            if (Input::get('email') != $user->email && Input::get('email') != '') {
                $rules['email'] = 'email|unique:users';
            }

            $input = Input::all();

            $validation = Validator::make($input, $rules);

            // A picat validarea?
            if ($validation->fails()) {
                Vsession::cadd('r', $validation->errors->first())->cflash('status');
            } else {
                foreach ($input as $key => $value) {
                    $input[$key] = ($value != '') ? trim(filter_var($value, FILTER_SANITIZE_STRING)) : null;
                }

                // Ce detalii vom modifica?

                if ($input['name'] != '') {
                    $user->name = $input['name'];
                }
                if ($input['email'] != '') {
                    $user->email = $input['email'];
                }
                if ($input['password'] != '') {
                    $user->password = $input['password1'];
                }
                // Stare cont
                $user->role = $input['role'];
                $user->verified = $input['verified'];
                $user->disabled = $input['disabled'];

                // Realizam actualizarea in baza de date
                $user->save();

                Vsession::cadd('g', __('site.user_edit_success'))->cflash('status');
                return Redirect::to_action('admin.users@edit/' . $id);
            }
        }

        return $this->get_edit($id, $user);
    }

    /**
     * Functia este apelata atunci cand se doreste stergerea unui utilizator din sistem
     *
     * @param null $id : Id-ul membrului care se doreste a se sterge
     * @return Laravel\Redirect : Catre ce pagina va fi redirectionat utilizatorul?
     */
    public function get_delete($id = null)
    {
        // Verificam daca id-ul este cel corect
        if ($id != null) {
            $id = trim(filter_var($id, FILTER_SANITIZE_NUMBER_INT));
        } else {
            return Redirect::to_action('admin.users@list');
        }

        // Nu ne vom puteas sterge pe noi insine
        if ($id == Auth::user()->id) {
            Vsession::cadd('r', __('site.user_delete_yourself'))->cflash('status');
            return Redirect::to_action('admin.users@list');
        }

        // Exista utilizatorul specificat prin id in sistem?
        if (!$this->user_exists($id)) {
            return Redirect::to_action('admin.users@list');
        }

        // Se face stergerea efectiva a utilizatorului din baza de date
        DB::table('users')
            ->where('id', '=', $id)
            ->delete();

        // Returnam un mesaj corespunzator
        Vsession::cadd('g', __('site.user_delete_success'))->cflash('status');
        return Redirect::to_action('admin.users@list');
    }

    /**
     * Functia este una interna si este folosita la intoarcerea tuturor utilizatorilor din sistem
     *
     * @return array : O lista cu toti utilizatorii din sistem
     */
    private function fetch_users()
    {
        $users = DB::table('users')
            ->get(array('id', 'username', 'name', 'email', 'role'));

        return $users;
    }

    /**
     * Functia verifica daca un membru specificat prin id-ul sau exista in sistem
     *
     * @param $id : Identificatorul membrului
     * @return bool : Adevarat numai daca exista
     */
    private function user_exists($id)
    {
        $user = DB::table('users')->find($id);

        return ($user !== null || !empty($user)) ? true : false;
    }

    /**
     * Functia intoarce datele unui membru din baza de date, dupa id-ul sau
     *
     * @param $id : Identificatorul membrului
     * @return array|null
     */
    private function fetch_user($id)
    {
        $user = DB::table('users')
            ->where('id', '=', (int)$id)
            ->take(1)
            ->get(array('id', 'username', 'name', 'email', 'role', 'verified', 'disabled'));

        return ($user != null) ? $user : null;
    }
}
