<?php

class Ajaxm_Controller extends Controller
{
    public $restful = true;

    /**
     * In constructor verificam sa avem o cerere Ajax
     */
    public function __construct()
    {
        parent::__construct();

        // Nu se permit decat cereri Ajax!!!
        if (!Request::ajax()) {
            die(__('site.no_ajax_request'));
        }
    }

    /**
     * Functia realizeaza procedura de autentificare a unui utilizator in sistem
     */
    public function post_check_login()
    {
        $input['username'] = Input::get('username');
        $input['password'] = Input::get('password');

        foreach($input as $field => $value) {
            $input[$field] = trim(filter_var($value, FILTER_SANITIZE_STRING));
        }

        $error = Auth::attempt($input);
        if ($error === true) {
            return View::make('main.blocks.menu');
        } else {
            return 0;
        }
    }

    /**
     * Functia este apelata in momentul in care un vizitator da clic pe butonul
     * de inregistrare...
     */
    public function post_register_user()
    {
        $return = array();
        
        $rules = array(
            'email' => 'required|email|unique:users',
            'name' => 'required|max:64',
            'username' => 'required|unique:users',
            'password1' => 'required|min:6|max:32',
            'password2' => 'required|same:password1'
        );

        $input = Input::all();
        $validation = Validator::make($input, $rules);

        if ($validation->fails()) {
            $return['myclass'] = 'warning';
            $return['message'] = $validation->errors->first();
        } else {

            // Adaugam utilizatorul in baza de date
            $user = new \Verify\Models\User;
            $user->username = $input['username'];
            $user->name = $input['name'];
            $user->email = $input['email'];
            $user->password = $input['password1'];
            $user->role = 0;
            $user->verified = 0;
            $user->disabled = 0;
            $user->save();

            $return['myclass'] = 'success';
            $return['message'] = Lang::line('main.register_form_success')->get();
        }

        return json_encode($return);
    }

    /**
     * Functia este apelata in momentul in care un vizitator doreste sa trimita
     * un mesaj prin intermediul formularului de contact...
     */
    public function post_contact_send()
    {
        $return = array();
        
        $rules = array(
            'email' => 'required|email',
            'name' => 'required|max:64',
            'message' => 'required|min:16|max:2048'
        );

        $input = Input::all();
        $validation = Validator::make($input, $rules);

        if ($validation->fails()) {
            $return['myclass'] = 'warning';
            $return['message'] = $validation->errors->first();
        } else {
            foreach ($input as $key => $value) {
                $input[$key] = ($value !== '') ? trim(filter_var($value, FILTER_SANITIZE_STRING)) : null;
            }

            $message = "Trimis de :\n" . $input['name'] . '(' . $input['email'] . ")\n\n";
            $message .= "Corp mesaj :\n" . $input['message'];

            // Trimitem mesajul prin mail folosind functia mail
            mail(Config::get('application.siteemail'), 'Mesaj de contact', $message);

            $return['myclass'] = 'success';
            $return['message'] = Lang::line('main.contact_form_success')->get();
        }

        return json_encode($return);
    }

    public function get_add_cart()
    {
        $id = (int)Input::get('item');

        $added = $this->add_if_not($id);

        if ( ! $added) {
            // Produsul exista deja in sistem, incrementam numarul lui...
            $more = $this->add_more($id);
        }

        return $this->count_products();
    }

    public function get_remove_cart()
    {
        $id = (int)Input::get('item');

        $this->remove_product($id);
    }

    /**
     * Functia sterge un produs din cosul de produse al unui utilizator...
     */
    private function remove_product($id)
    {
        $array = array();
        $ip = Request::ip();
        if (Session::has('cart')) {
            $array = Session::get('cart');
        }

        // Verificam daca exista o sesiune pentru IP-ul respectiv
        if (isset($array[$ip])) {
            // Verificam daca nu exista in cos...
            if (isset($array[$ip][$id])) {
                unset($array[$ip][$id]);
            }
        }
        
        // Actualizam datele prezente in sesiune...
        $this->put_data('cart', $array);
    }

    /**
     * Numaram cate produse sunt in cosul de cumparaturi...
     */
    private function count_products()
    {
        $count = 0;
        $ip = Request::ip();

        if (Session::has('cart')) {
            $array = Session::get('cart');

            if (isset($array[$ip])) {
                $ar = $array[$ip];
                foreach ($ar as $key => $val) {
                    $count += $val;
                }
            }
        }

        return $count;
    }

    /**
     * Functia adauga un produs intr-un cos doar daca nu exista deja acolo...
     */
    private function add_if_not($id)
    {
        $array = array();
        $ip = Request::ip();
        if (Session::has('cart')) {
            $array = Session::get('cart');
        }

        // Verificam daca exista o sesiune pentru IP-ul respectiv
        if (isset($array[$ip])) {
            // Verificam daca nu exista in cos...
            if ( ! isset($array[$ip][$id])) {
                // Il punem in cos...
                $array[$ip][$id] = 1;
                $this->put_data('cart', $array);
                return true;
            } else {
                return false;
            }
        } else {
            // Cream un subarbore...
            $array[$ip] = array($id => 1);
            $this->put_data('cart', $array);
            return true;
        }
    }

    /**
     * Functia adauga un produs de mai multe ori in acelasi cos...
     */
    private function add_more($id)
    {
        $ip = Request::ip();
        if (Session::has('cart')) {
            $array = Session::get('cart');
            
            $array[$ip][$id]++;
            $this->put_data('cart', $array);

            return true;
        }
    
        return false;
    }

    /**
     * Functia pune anumite date intr-o sesiune, folosind functii Laravel
     */
    private function put_data($name, $value)
    {
        Session::forget($name);
        Session::put($name, $value);
    }

}
