<?php

class Base_Controller extends Controller
{
    // Toate controllerele aplicatiei vor fi de tip restful
	public $restful = true;

    /**
     * In constructor, trebuie neaparat apelat constructorul de baza al unui controller.
     * Tot aici vom initia variabilele si evenimentele globale ale aplicatiei.
     */
    public function __construct()
	{
		parent::__construct();

		// Se incarca setarile din baza de date
		$settings = $this->load_settings('settings');

		// Dupa ce se incarca se vor seta...
		if($settings) {
            $this->set_settings($settings);
        }
	}

    /**
     * Incercarea de a accesa alte functii din controllerul de baza, va conduce la
     * primirea unui raspuns 404 - Page Not Found.
     * Functie necesara pentru sporirea securitatii aplicatiei
     *
     * @param $method
     * @param $parameters
     * @return Laravel\Response
     */
    public function __call($method, $parameters)
	{
		return Response::error('404');
	}

    /**
     * Functia incarca de pe disc sau din baza de date o serie de setari ale aplicatiei
     *
     * @param $table
     * @return array|mixed
     */
    private function load_settings($table)
	{
        // Exista datele in cache?
        $settings = Cache::get($table . '.cache');
        if ($settings != null) {
            return $settings;
        } else {
            // Daca nu exista in cache va trebui sa facem o interogare SQL
		    $settings = DB::table($table)->get(array('field', 'name', 'value'));
            // Setarile le vom salva si in cache pentru a eficientiza modul de lucru
            Cache::forever($table . '.cache', $settings);
		    return $settings;
        }
	}

    /**
     * Dupa ce le-am incarcat, aceste setari trebuie actualizate in cadrul aplicatiei
     *
     * @param array $settings
     */
    private function set_settings($settings = array())
	{
        // Setam, pe rand, toate setarile aplicatiei
		foreach ($settings as $setting) {
			Config::set($setting->name, $setting->value);
		}
	}
}

?>