<?php

class Login_Controller extends Base_Controller
{
    public function get_index()
    {
        // Realizam o redirectionare catre pagina de autentificare
        return Redirect::to_action('login@in');
    }

    public function get_in()
    {
        // Verificam sa nu fim deja autentificati...
        if ( ! Auth::guest()) {
            return $this->already_auth();
        }

        // Daca nu suntem autentificati, va trebui sa ne autentificam
        return View::make('layout.login');
    }

    public function post_in()
    {
        $this->filter('before', 'csrf');

        if (Input::get('login')) {
            $rules = array(
                'username' => 'required|max:32',
                'password' => 'required'
            );

            $input = Input::all();

            $validation = Validator::make($input, $rules);

            if ($validation->fails()) {
                Vsession::cadd('r', $validation->errors->first())->cflash('status');
            } else {
                foreach ($input as $field => $value) {
                    $input[$field] = trim(filter_var($value, FILTER_SANITIZE_STRING));
                }

                $credentials = array(
                    'username' => $input['username'],
                    'password' => $input['password']
                );

                $error = Auth::attempt($credentials);
                Vsession::cadd('r', $error)->cflash('status');
                if ($error === true) {
                    // Realizam o redirectionare
                    return $this->already_auth();
                } else {
                    // Setam eroarea, daca a aparut una
                    Vsession::cadd('r', __('site.login_error' . $error))->cflash('status');
                }
            }
        }

        return $this->get_in();
    }

    public function get_out()
    {
        Auth::logout();

        return Redirect::to_action('index@home');
    }

    private function already_auth()
    {
        return Redirect::to_action('admin.users@list');
    }
}

?>